@@ -1,6 +1,7 @@
 import inspect
 import logging
 import random
+import re
 import sys
 from collections import defaultdict
 from contextlib import redirect_stdout
@@ -21,6 +22,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import pytz
+from dateutil.relativedelta import relativedelta
 from humanize import naturaltime
 from yaml import load
 
@@ -284,7 +286,14 @@ class Turbot(discord.Client):
                 self._users_data = pd.read_csv(self.users_file)
             except FileNotFoundError:
                 self._users_data = pd.DataFrame(
-                    columns=["author", "hemisphere", "timezone"]
+                    columns=[
+                        "author",
+                        "hemisphere",
+                        "timezone",
+                        "island",
+                        "friend",
+                        "fruit",
+                    ]
                 )
         return self._users_data
 
@@ -410,40 +419,27 @@ class Turbot(discord.Client):
         )
         return last.iloc[0] if last.any() else None
 
-    def _get_user_prefs(self, author_id):
+    def get_user_prefs(self, author_id):
         users = self.load_users()
         row = users[users.author == author_id].tail(1)
         if row.empty:
             return {}
-        return row.to_dict(orient="records")[0]
-
-    def get_user_hemisphere(self, author_id):
-        prefs = self._get_user_prefs(author_id)
-        if (
-            not prefs
-            or "hemisphere" not in prefs
-            or not prefs["hemisphere"]
-            or not isinstance(prefs["hemisphere"], str)
-        ):
-            return None
-        return prefs["hemisphere"]
-
-    def get_user_timezone(self, author_id):
-        prefs = self._get_user_prefs(author_id)
-        if (
-            not prefs
-            or "timezone" not in prefs
-            or not prefs["timezone"]
-            or not isinstance(prefs["timezone"], str)
-        ):
-            return pytz.UTC
-        return pytz.timezone(prefs["timezone"])
+        data = row.to_dict(orient="records")[0]
+        prefs = {}
+        for column in users.columns:
+            if data and column in data and data[column] and isinstance(data[column], str):
+                if column == "timezone":
+                    prefs[column] = pytz.timezone(data[column])
+                else:
+                    prefs[column] = data[column]
+        return prefs
 
     def to_usertime(self, author_id, dt):
+        user_timezone = self.get_user_prefs(author_id).get("timezone", pytz.UTC)
         if hasattr(dt, "tz_convert"):  # pandas-datetime-like objects
-            return dt.tz_convert(self.get_user_timezone(author_id))
+            return dt.tz_convert(user_timezone)
         elif hasattr(dt, "astimezone"):  # python-datetime-like objects
-            return dt.astimezone(self.get_user_timezone(author_id))
+            return dt.astimezone(user_timezone)
         else:  # pragma: no cover
             logging.warning(f"can't convert tz on {dt} for user {author_id}")
             return dt
@@ -1191,7 +1187,7 @@ class Turbot(discord.Client):
         yours = yours.sort_values(by=["timestamp"])
 
         # convert all timestamps to the target user's timezone
-        target_timezone = self.get_user_timezone(target_id)
+        target_timezone = self.get_user_prefs(target_id).get("timezone", pytz.UTC)
         yours["timestamp"] = yours.timestamp.dt.tz_convert(target_timezone)
 
         recent_buy = yours[yours.kind == "buy"].tail(1)
@@ -1219,6 +1215,34 @@ class Turbot(discord.Client):
         url = f"{self.base_prophet_url}{query}"
         return s("predict", name=target_name, url=url), None
 
+    def friend_command(self, channel, author, params):
+        """
+        Set your friend code. | <code>
+        """
+        if not params:
+            return s("friend_no_params"), None
+
+        code = re.sub("[^0-9]", "", "".join(params).replace("-", ""))
+        if len(code) != 12 or not code.isdigit():
+            return s("friend_invalid"), None
+
+        self.save_user_pref(author, "friend", code)
+        return s("friend", name=author), None
+
+    def fruit_command(self, channel, author, params):
+        """
+        Set your island's native fruit. | [apple|cherry|orange|peach|pear]
+        """
+        if not params:
+            return s("fruit_no_params"), None
+
+        fruit = params[0].lower()
+        if fruit not in ["apple", "cherry", "orange", "peach", "pear"]:
+            return s("fruit_invalid"), None
+
+        self.save_user_pref(author, "fruit", fruit)
+        return s("fruit", name=author), None
+
     def hemisphere_command(self, channel, author, params):
         """
         Set your hemisphere. | [Northern|Southern]
@@ -1248,6 +1272,17 @@ class Turbot(discord.Client):
         self.save_user_pref(author, "timezone", zone)
         return s("timezone", name=author), None
 
+    def island_command(self, channel, author, params):
+        """
+        Set your island name. | <name>
+        """
+        if not params:
+            return s("island_no_params"), None
+
+        island = " ".join(params)  # allow spaces in island names
+        self.save_user_pref(author, "island", island)
+        return s("island", name=author), None
+
     def count_command(self, channel, author, params):
         """
         Provides a count of the number of pieces of collectables for the comma-separated
@@ -1342,14 +1377,15 @@ class Turbot(discord.Client):
 
     def _creatures(self, *_, author, params, kind, source, force_text=False):
         """The fish and bugs commands are so similar; I factored them out to a helper."""
-        hemisphere = self.get_user_hemisphere(author.id)
+        hemisphere = self.get_user_prefs(author.id).get("hemisphere", None)
         if not hemisphere:
             return s("no_hemisphere")
 
         now = self.to_usertime(author.id, datetime.now(pytz.utc))
         this_month = now.strftime("%b").lower()
-        next_month = (now + timedelta(days=33)).strftime("%b").lower()
-        last_month = (now - timedelta(days=33)).strftime("%b").lower()
+        first_this_month = now.replace(day=1)
+        last_month = (first_this_month - timedelta(days=1)).strftime("%b").lower()
+        next_month = (first_this_month + relativedelta(months=1)).strftime("%b").lower()
         available = source[(source.hemisphere == hemisphere) & (source[this_month] == 1)]
 
         def details(row):
@@ -1376,7 +1412,8 @@ class Turbot(discord.Client):
             return lines
 
         if params:
-            search = " ".join(params)
+            user_input = " ".join(params)
+            search = user_input.lower()
             if search == "leaving":
                 found = available[available[next_month] == 0]
             elif search == "arriving":
@@ -1385,7 +1422,7 @@ class Turbot(discord.Client):
                 found = available[available.name.str.contains(search)]
 
             if found.empty:
-                return s(f"{kind}_none_found", search=search)
+                return s(f"{kind}_none_found", search=user_input)
             elif force_text or len(found) > EMBED_LIMIT:
                 available = found  # fallback to the less detailed, text only, response
             else:
@@ -1453,6 +1490,34 @@ class Turbot(discord.Client):
             None,
         )
 
+    def _info_embed(self, user):
+        prefs = self.get_user_prefs(user.id)
+
+        embed = discord.Embed(title=user.name)
+        embed.set_thumbnail(url=user.avatar_url)
+
+        island = prefs.get("island", "Not set")
+        embed.add_field(name="Island", value=island)
+
+        hemisphere = prefs.get("hemisphere", "Not set").title()
+        embed.add_field(name="Hemisphere", value=hemisphere)
+
+        now = self.to_usertime(user.id, datetime.now(pytz.UTC))
+        current_time = now.strftime("%I:%M %p %Z")
+        embed.add_field(name="Current time", value=current_time)
+
+        fruit = prefs.get("fruit", "Not set").title()
+        embed.add_field(name="Native fruit", value=fruit)
+
+        code = prefs.get("friend", None)
+        if code:
+            code_str = f"SW-{code[0:4]}-{code[4:8]}-{code[8:]}"
+            embed.add_field(name="Friend code", value=code_str)
+        else:
+            embed.add_field(name="Friend code", value="Not set")
+
+        return embed
+
     def info_command(self, channel, author, params):
         """
         Gives you information on a user. | [user]
@@ -1463,21 +1528,12 @@ class Turbot(discord.Client):
         users = self.load_users()
         for _, row in users.iterrows():
             user_id = row["author"]
-            name = discord_user_name(channel, user_id)
-            if not name:
+            user_name = discord_user_name(channel, user_id)
+            if not user_name:
                 continue
-
-            if name.lower().find(params[0].lower()) != -1:
-                now = self.to_usertime(user_id, datetime.now(pytz.utc))
-                return (
-                    s(
-                        "info",
-                        hemisphere=self.get_user_hemisphere(user_id).title(),
-                        time=now.strftime("%I:%M %p %Z"),
-                        name=name,
-                    ),
-                    None,
-                )
+            if user_name.lower().find(params[0].lower()) != -1:
+                user = discord_user_from_name(channel, user_name)
+                return self._info_embed(user), None
 
         return s("info_not_found"), None
 
